@@ -1,19 +1,19 @@
 # -*- coding: utf-8 -*-
 
 #  smartspaces.py
-#  
+#
 #  Copyright (C) 2006 - Steve Frécinaux
-#  
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
-#   
+#
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU General Public License for more details.
-#   
+#
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330,
@@ -54,46 +54,56 @@ class SmartSpacesViewHelper(object):
     def on_notify(self, view, pspec):
         self.update_active()
 
+    def get_real_indent_width(view):
+        indent_width = view.get_indent_width()
+
+        if indent_width < 0:
+             indent_width = view.get_tab_width()
+
+        return indent_width
+
     def on_key_press_event(self, view, event):
-        # Only take care of backspace
-        if event.keyval != gtk.keysyms.BackSpace:
+        # Only take care of backspace and shift+backspace
+        mods = gtk.accelerator_get_default_mod_mask()
+        if event.keyval != gtk.keysyms.BackSpace or \
+	       event.state & mods != 0 and event.state & mods != gtk.gdk.SHIFT_MASK:
             return False
-        
+
         doc = view.get_buffer()
         if doc.get_has_selection():
             return False
-        
+
         cur = doc.get_iter_at_mark(doc.get_insert())
         offset = cur.get_line_offset()
-        
+
         if offset == 0:
             # We're at the begining of the line, so we can't obviously
             # unindent in this case
             return False
 
         start = cur.copy()
-        start.backward_char()
+        prev = cur.copy()
+        prev.backward_char()
+
+        # If the previus chars are spaces, try to remove
+        # them until the previus tab stop
+        max_move = offset % get_real_indent_width(view)
+        if max_move == 0:
+            max_move = get_real_indent_width(view)
 
-        # If the previous char is a tab, we should just remove it
-        if start.get_char() == '\t':
-            doc.begin_user_action()
-            doc.delete(start, cur)
-            doc.end_user_action()
-            return True
-        
-        # Otherwise, check how many spaces we're able to remove
-        max_move = (offset - 1) % view.get_tabs_width() + 1
         moved = 0
-        while moved < max_move and start.get_char() == ' ':
+        while moved < max_move and prev.get_char() == ' ':
             start.backward_char()
             moved += 1
-        start.forward_char()
-        
+            if not prev.backward_char():
+                # we reached the start of the buffer
+                break
+
         if moved == 0:
-            # The iterator hasn't moved, so there is nothing to remove
+            # The iterator hasn't moved, it was not a space
             return False
 
-        # Actually delete the spaces        
+        # Actually delete the spaces
         doc.begin_user_action()
         doc.delete(start, cur)
         doc.end_user_action()
@@ -109,24 +119,24 @@ class SmartSpacesPlugin(gedit.Plugin):
     def add_helper(self, view):
         helper = SmartSpacesViewHelper(view)
         view.set_data(self.VIEW_DATA_KEY, helper)
-    
+
     def remove_helper(self, view):
         view.get_data(self.VIEW_DATA_KEY).deactivate()
         view.set_data(self.VIEW_DATA_KEY, None)
-                    
+
     def activate(self, window):
         for view in window.get_views():
             self.add_helper(view)
-            
+
         handler_id = window.connect("tab-added",
                                     lambda w, t: self.add_helper(t.get_view()))
         window.set_data(self.WINDOW_DATA_KEY, handler_id)
-    
+
     def deactivate(self, window):
         handler_id = window.get_data(self.WINDOW_DATA_KEY)
         window.disconnect(handler_id)
         window.set_data(self.WINDOW_DATA_KEY, None)
-        
+
         for view in window.get_views():
             self.remove_helper(view)
 
